"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const common_tags_1 = require("common-tags");
const ts = tslib_1.__importStar(require("typescript"));
const cli_1 = require("../../lib/cli");
const Layout = tslib_1.__importStar(require("../../lib/layout"));
const nexus_logger_1 = require("../../lib/nexus-logger");
const own_package_1 = require("../../lib/own-package");
const Plugin = tslib_1.__importStar(require("../../lib/plugin"));
const process_1 = require("../../lib/process");
const tsc_1 = require("../../lib/tsc");
const watcher_1 = require("../../lib/watcher");
const start_1 = require("../../runtime/start");
const log = nexus_logger_1.rootLogger.child('dev');
const DEV_ARGS = {
    '--inspect-brk': String,
    '--entrypoint': String,
    '-e': '--entrypoint',
    '--help': Boolean,
    '-h': '--help',
};
class Dev {
    async parse(argv) {
        var _a, _b;
        const args = cli_1.arg(argv, DEV_ARGS);
        if (cli_1.isError(args)) {
            process_1.fatal(args.message);
        }
        if (args['--help']) {
            return this.help();
        }
        /**
         * Load config before loading plugins which may rely on env vars being defined
         */
        const entrypointPath = args['--entrypoint'];
        const layout = await Layout.create({ entrypointPath });
        const pluginEntrypoints = await Plugin.getUsedPlugins(layout);
        const worktimePlugins = await Plugin.importAndLoadWorktimePlugins(pluginEntrypoints, layout);
        for (const p of worktimePlugins) {
            await ((_b = (_a = p.hooks.dev).onStart) === null || _b === void 0 ? void 0 : _b.call(_a));
        }
        log.info('start', { version: own_package_1.ownPackage.version });
        const layoutPlugin = {
            build: {},
            create: {},
            generate: {},
            dev: {
                addToWatcherSettings: {},
                async onBeforeWatcherStartOrRestart(change) {
                    if (change.type === 'init' ||
                        change.type === 'add' ||
                        change.type === 'addDir' ||
                        change.type === 'unlink' ||
                        change.type === 'unlinkDir') {
                        log.trace('analyzing project layout');
                        const layout = await Layout.create({ entrypointPath });
                        return {
                            environmentAdditions: Layout.saveDataForChildProcess(layout),
                        };
                    }
                },
            },
        };
        const startModule = start_1.createStartModuleContent({
            internalStage: 'dev',
            runtimePluginManifests: [],
            layout,
            absoluteModuleImports: true,
        });
        const transpiledStartModule = tsc_1.transpileModule(startModule, {
            target: ts.ScriptTarget.ES5,
            module: ts.ModuleKind.CommonJS,
        });
        const watcher = await watcher_1.createWatcher({
            entrypointScript: transpiledStartModule,
            sourceRoot: layout.sourceRoot,
            cwd: process.cwd(),
            plugins: [layoutPlugin].concat(worktimePlugins.map((p) => p.hooks)),
            inspectBrk: args['--inspect-brk'],
        });
        await watcher.start();
    }
    help() {
        return common_tags_1.stripIndent `
        Usage: nexus dev [flags]
  
        Develop your application in watch mode
  
        Flags:
          -e, --entrypoint    Custom entrypoint to your app (default: app.ts)
          -h,       --help    Show this help message
      `;
    }
}
exports.Dev = Dev;
//# sourceMappingURL=dev.js.map