"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Path = tslib_1.__importStar(require("path"));
const Layout = tslib_1.__importStar(require("../../lib/layout"));
const tsc_1 = require("../../lib/tsc");
const nexus_logger_1 = require("../nexus-logger");
const utils_1 = require("../utils");
const extractor_1 = require("./extractor");
const typegen_1 = require("./typegen");
const log = nexus_logger_1.rootLogger.child('add-to-context-extractor');
/**
 * Run the extractor in a work if possible. For example in Node 10 workers are
 * not available by default. If workers are not available then extraction falls
 * back to running in this process, possibly blocking with with intensive CPU work.
 */
function runAddToContextExtractorAsWorkerIfPossible(layoutData) {
    log.trace('starting context type extraction');
    let hasWorkerThreads = utils_1.areWorkerThreadsAvailable();
    if (hasWorkerThreads) {
        log.trace('Worker threads available');
        return runAddToContextExtractorAsWorker(layoutData);
    }
    else {
        log.trace('Worker threads unavailable. Fallbacking to main process');
        const layout = Layout.createFromData(layoutData);
        const builder = tsc_1.createTSProgram(layout, { withCache: true });
        return runAddToContextExtractorAsPromise(builder.getProgram());
    }
}
exports.runAddToContextExtractorAsWorkerIfPossible = runAddToContextExtractorAsWorkerIfPossible;
/**
 * Run the pure extractor and then write results to a typegen module.
 */
async function runAddToContextExtractorAsPromise(program) {
    const contextTypes = extractor_1.extractContextTypes(program);
    await typegen_1.writeContextTypeGenFile(contextTypes);
    return contextTypes;
}
exports.runAddToContextExtractorAsPromise = runAddToContextExtractorAsPromise;
/**
 * Run the extractor in a worker.
 */
function runAddToContextExtractorAsWorker(layoutData) {
    return new Promise((resolve, reject) => {
        // avoid import error in node 10.x
        const { Worker } = require('worker_threads');
        const worker = new Worker(Path.join(__dirname, 'worker.js'), {
            workerData: {
                layoutData,
            },
        });
        worker.once('online', () => {
            log.trace('worker online');
        });
        worker.once('exit', (exitCode) => {
            log.trace('worker exited', { exitCode });
        });
        worker.once('message', (contextTypes) => {
            log.trace('worker finished context type extraction', { contextTypes });
            // Let the Node.js main thread exit, even though the Worker
            // is still running:
            worker.unref();
            resolve(contextTypes);
        });
        worker.on('error', (error) => {
            log.warn('We could not extract your context types from `schema.addToContext`', { error });
            reject(error);
        });
    });
}
exports.runAddToContextExtractorAsWorker = runAddToContextExtractorAsWorker;
//# sourceMappingURL=add-to-context-extractor.js.map