/// <reference types="pino" />
import * as Level from './level';
import * as Logger from './logger';
import * as Output from './output';
import * as Pino from './pino';
export declare type SettingsData = Readonly<{
    level: Level.Level;
    pretty: Readonly<{
        enabled: boolean;
        color: boolean;
        levelLabel: boolean;
        timeDiff: boolean;
    }>;
    output: Output.Output;
}>;
export declare type SettingsInput = {
    /**
     * Set the level of this and all descedent loggers. This level setting has
     * highest precedence of all logger level configuration tiers.
     *
     * The level config takes the first value found, searching tiers as follows:
     *
     *  1. logger instance setting
     *  2. logger constructor setting
     *  3. LOG_LEVEL environment variable setting
     *  3. NODE_ENV=production -> info
     *  4. otherwise -> debug
     */
    level?: Level.Level;
    /**
     * Control pretty mode.
     *
     * Shorthands:
     *
     *  - `true` is shorthand for `{ enabled: true }`
     *  - `false` is shorthand for `{ enabled: false }`
     *
     * When `undefined` pretty takes the first value found, in order:
     *
     *  1. `process.env.LOG_PRETTY` (admits case insensitive: `true` | `false`)
     *  2. `process.stdout.isTTY`
     */
    pretty?: boolean | {
        /**
         * Disable or enable pretty mode.
         *
         * When `undefined` pretty takes the first value found, in order:
         *
         *  1. `process.env.LOG_PRETTY` (admits case insensitive: `true` | `false`)
         *  2. `process.stdout.isTTY`
         */
        enabled?: boolean;
        /**
         * Should logs be colored?
         *
         * @default `true`
         *
         * Disabling can be useful when pretty logs are going to a destination that
         * does not support rendering ANSI color codes (consequence being very
         * difficult to read content).
         */
        color?: boolean;
        /**
         * Should logs include the level label?
         *
         * @default `false`
         *
         * Enable this if understanding the level of a log is important to you
         * and the icon+color system is insufficient for you to do so. Can be
         * helpful for newcomers or a matter of taste for some.
         */
        levelLabel?: boolean;
        /**
         * Should the logs include the time between it and previous log?
         *
         * @default `true`
         */
        timeDiff?: boolean;
    };
};
declare type Settings = SettingsData & {
    (newSettings: SettingsInput): RootLogger;
};
export declare type Options = SettingsInput & {
    output?: Output.Output;
    name?: string;
};
export declare type RootLogger = Logger.Logger & {
    settings: Settings;
};
export declare type State = {
    pino: Pino.Logger;
};
/**
 * Create a root logger.
 */
export declare function create(opts?: Options): RootLogger;
export {};
//# sourceMappingURL=root-logger.d.ts.map