"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const NexusSchema = tslib_1.__importStar(require("@nexus/schema"));
const common_tags_1 = require("common-tags");
const fs = tslib_1.__importStar(require("fs-jetpack"));
const Lo = tslib_1.__importStar(require("lodash"));
const path_1 = require("path");
const layout_1 = require("../../lib/layout/layout");
const logger_1 = require("./logger");
const log = logger_1.log.child('settings');
function changeSettings(state, newSettings) {
    var _a, _b;
    if (newSettings.nullable) {
        if (state.nullable === undefined) {
            state.nullable = {};
        }
        Lo.merge(state.nullable, newSettings.nullable);
    }
    if (newSettings.generateGraphQLSDLFile) {
        state.generateGraphQLSDLFile = newSettings.generateGraphQLSDLFile;
    }
    if (newSettings.rootTypingsGlobPattern) {
        state.rootTypingsGlobPattern = newSettings.rootTypingsGlobPattern;
    }
    if (newSettings.connections) {
        state.connections = (_a = state.connections) !== null && _a !== void 0 ? _a : {};
        const _c = newSettings.connections, { types } = _c, connectionPluginConfig = tslib_1.__rest(_c, ["types"]);
        if (types) {
            state.connections.types = (_b = state.connections.types) !== null && _b !== void 0 ? _b : {};
            Object.assign(state.connections.types, types);
        }
        Object.assign(state.connections, connectionPluginConfig);
    }
}
exports.changeSettings = changeSettings;
exports.NEXUS_DEFAULT_TYPEGEN_PATH = fs.path('node_modules', '@types', 'typegen-nexus', 'index.d.ts');
function mapSettingsToNexusSchemaConfig(frameworkPlugins, settings) {
    var _a, _b, _c, _d, _e, _f;
    const baseConfig = {
        nonNullDefaults: {
            input: !((_b = (_a = settings === null || settings === void 0 ? void 0 : settings.nullable) === null || _a === void 0 ? void 0 : _a.inputs) !== null && _b !== void 0 ? _b : true),
            output: !((_d = (_c = settings === null || settings === void 0 ? void 0 : settings.nullable) === null || _c === void 0 ? void 0 : _c.outputs) !== null && _d !== void 0 ? _d : true),
        },
        outputs: {
            schema: getOutputSchemaPath(settings),
            typegen: exports.NEXUS_DEFAULT_TYPEGEN_PATH,
        },
        typegenAutoConfig: {
            sources: [],
        },
        shouldGenerateArtifacts: shouldGenerateArtifacts(),
        shouldExitAfterGenerateArtifacts: shouldExitAfterGenerateArtifacts(),
        types: [],
        plugins: [],
    };
    baseConfig.plugins.push(...processConnectionsConfig(settings));
    // Merge the plugin nexus plugins
    for (const frameworkPlugin of frameworkPlugins) {
        const schemaPlugins = (_f = (_e = frameworkPlugin.schema) === null || _e === void 0 ? void 0 : _e.plugins) !== null && _f !== void 0 ? _f : [];
        baseConfig.plugins.push(...schemaPlugins);
    }
    const finalConfig = withAutoTypegenConfig(baseConfig, frameworkPlugins);
    log.trace('config built', { config: finalConfig });
    return finalConfig;
}
exports.mapSettingsToNexusSchemaConfig = mapSettingsToNexusSchemaConfig;
function withAutoTypegenConfig(nexusConfig, plugins) {
    var _a;
    // Integrate plugin typegenAutoConfig contributions
    const typegenAutoConfigFromPlugins = {};
    for (const p of plugins) {
        if ((_a = p.schema) === null || _a === void 0 ? void 0 : _a.typegenAutoConfig) {
            Lo.merge(typegenAutoConfigFromPlugins, p.schema.typegenAutoConfig);
        }
    }
    const typegenAutoConfigObject = Lo.merge({}, typegenAutoConfigFromPlugins, nexusConfig.typegenAutoConfig);
    nexusConfig.typegenAutoConfig = undefined;
    function contextTypeContribSpecToCode(ctxTypeContribSpec) {
        return common_tags_1.stripIndents `
      interface Context {
        ${Object.entries(ctxTypeContribSpec)
            .map(([name, type]) => {
            // Quote key name to handle case of identifier-incompatible key names
            return `'${name}': ${type}`;
        })
            .join('\n')}
      }
    `;
    }
    // Our use-case of multiple context sources seems to require a custom
    // handling of typegenConfig. Opened an issue about maybe making our
    // curreent use-case, fairly basic, integrated into the auto system, here:
    // https://github.com/prisma-labs/nexus/issues/323
    nexusConfig.typegenConfig = async (schema, outputPath) => {
        const configurator = await NexusSchema.core.typegenAutoConfig(typegenAutoConfigObject);
        const config = await configurator(schema, outputPath);
        // Initialize
        config.imports.push('interface Context {}');
        config.imports.push(common_tags_1.stripIndent `
      declare global {
        interface NexusContext extends Context {}
      }
    `);
        config.contextType = 'NexusContext';
        // Integrate plugin context contributions
        for (const p of plugins) {
            if (!p.context)
                continue;
            if (p.context.typeGen.imports) {
                config.imports.push(...p.context.typeGen.imports.map((im) => `import * as ${im.as} from '${im.from}'`));
            }
            config.imports.push(contextTypeContribSpecToCode(p.context.typeGen.fields));
        }
        config.imports.push("import * as Logger from 'nexus/dist/lib/logger'", contextTypeContribSpecToCode({
            log: 'Logger.Logger',
        }));
        config.nexusSchemaImportId = 'nexus/components/schema';
        log.trace('built up Nexus typegenConfig', { config });
        return config;
    };
    return nexusConfig;
}
function shouldGenerateArtifacts() {
    return process.env.NEXUS_SHOULD_GENERATE_ARTIFACTS === 'true'
        ? true
        : process.env.NEXUS_SHOULD_GENERATE_ARTIFACTS === 'false'
            ? false
            : Boolean(!process.env.NODE_ENV || process.env.NODE_ENV === 'development');
}
exports.shouldGenerateArtifacts = shouldGenerateArtifacts;
function shouldExitAfterGenerateArtifacts() {
    return process.env.NEXUS_SHOULD_EXIT_AFTER_GENERATE_ARTIFACTS === 'true' ? true : false;
}
exports.shouldExitAfterGenerateArtifacts = shouldExitAfterGenerateArtifacts;
/**
 * Process the schema connection settings into nexus schema relay connection
 * plugins.
 */
function processConnectionsConfig(settings) {
    if (settings.connections === undefined) {
        return [
            defaultConnectionPlugin({
                nexusSchemaImportId: 'nexus/components/schema',
            }),
        ];
    }
    const instances = [];
    const _a = settings.connections, { default: defaultTypeConfig } = _a, customTypesConfig = tslib_1.__rest(_a, ["default"]);
    for (const [name, config] of Object.entries(customTypesConfig)) {
        if (config) {
            instances.push(NexusSchema.connectionPlugin(Object.assign({ nexusSchemaImportId: 'nexus/components/schema', nexusFieldName: name }, config)));
        }
    }
    if (defaultTypeConfig) {
        instances.push(defaultConnectionPlugin(defaultTypeConfig));
    }
    return instances;
}
function defaultConnectionPlugin(configBase) {
    return NexusSchema.connectionPlugin(Object.assign(Object.assign({}, configBase), { nexusFieldName: 'connection' }));
}
function getOutputSchemaPath(settings) {
    if (process.env.NEXUS_STAGE !== 'dev') {
        return false;
    }
    const layout = layout_1.mustLoadDataFromParentProcess(); // Using layout at runtime and in "prod"
    // Process graphQL SDL output setting
    let outputSchema;
    if (settings.generateGraphQLSDLFile === undefined || settings.generateGraphQLSDLFile === false) {
        outputSchema = false;
    }
    else {
        if (path_1.isAbsolute(settings.generateGraphQLSDLFile)) {
            outputSchema = settings.generateGraphQLSDLFile;
        }
        else {
            outputSchema = layout.projectPath(settings.generateGraphQLSDLFile);
        }
    }
    return outputSchema;
}
//# sourceMappingURL=settings.js.map